import * as chalk from 'chalk'
import { Questions, prompt } from 'inquirer'

export const username: Questions = {
  type: 'input',
  name: 'username',
  message: chalk.bold('Username:')
}
export const password: Questions = {
  type: 'password',
  name: 'password',
  message: chalk.bold('Password:')
}

export const entry: Questions = {
  type: 'confirm',
  name: 'entry',
  message: `Detectado operação de ${chalk.bold.italic.magenta('entrada')}, confirmar?`,
  default: true
}

export const exit: Questions = {
  type: 'confirm',
  name: 'exit',
  message: `Detectado operação de ${chalk.bold.italic.magenta('saída')}, confirmar?`,
  default: true
}

export async function accessSubtask (description: string) {
  const {response} = await prompt({
    type: 'list',
    name: 'response',
    message: `A tarefa '${chalk.cyan(description)}' permite registro, porém também contém sub-tarefas, deseja acessa-las?'`,
    choices: [{name: 'Registrar tarefa', value: 'N'}, {name: 'Acessar sub-tarefas', value: 'S'}]
  })
  return response === 'S'
}

export async function describeTask (): Promise<string> {
  const response = await prompt({
    type: 'input',
    name: 'description',
    message: 'Descreva o realizado na tarefa:',
    validate: (input => Boolean(input.trim()) || chalk.red('Descrição é obrigatória'))
  })
  return response.description
}

export async function timeTask (time: number) {
  const totalHours = Math.floor(time / 60)
  const totalMinutes = time % 60
  const regex = /^\s*((\d{1,2})h){0,1}\s*((\d{1,2})min){0,1}\s*$/
  const response = await prompt({
    type: 'input',
    name: 'string',
    message: `Tempo gasto na tarefa, até o máximo de ${chalk.underline.bold.italic.blue(`${totalHours}h ${totalMinutes}min`)}:`,
    validate: (input => {
      if (!input.trim()) return chalk.red('Tempo é obrigatório')
      const [, hours] = input.match(/(\d*)h/) || [undefined, 0]
      if (hours > 23) return chalk.red('Não é possível cadastrar uma atividade com mais de 23 horas')
      const [, minutes] = input.match(/(\d*)min/) || [undefined, 0]
      if (minutes > 59) return chalk.red('Não é possível cadastrar uma atividade com mais de 59 minutos')
      const inputTime = (Number(hours) * 60) + Number(minutes)
      if (time - inputTime < 0) return chalk.red('Tempo informado ultrapassa o tempo limite')
      return regex.test(input) || chalk.red('Formatação inválida')
    })
  })
  const str: string = response.string
  const [, hours] = str.match(/(\d*)h/) || [undefined, 0]
  const [, minutes] = str.match(/(\d*)min/) || [undefined, 0]
  return (Number(hours) * 60) + Number(minutes)
}

export const confirmExit: Questions = {
  type: 'confirm',
  name: 'confirmExit',
  message: `Tem certeza que deseja registrar operação de ${chalk.magenta('saída')}?`,
  default: true
}
