import * as program from 'commander'
import { prompt } from 'inquirer'
import { UcRegistrarPonto, RegistroPonto, Item, Domain, Tarefa } from '../../../curio'
import { exit, accessSubtask, describeTask, timeTask, confirmExit } from '../../../constants/questions'
import { BusinessError, SystemError, ConnectionError } from 'curiojs'
import * as moment from 'moment'
import * as chalk from 'chalk'
import * as fuzzy from 'fuzzy'
import * as makeSpinner from 'ora'

import promptProjeto from './projeto'
import promptTarefa from './tarefa'

const {magenta, blue, cyan, red, green} = chalk

const Table: any = require('cli-table')

export default async function saida (dominio: Domain, ultimoRegistro: RegistroPonto, registro: RegistroPonto, ucRegistrarPonto: UcRegistrarPonto) {
  if (!(await prompt(exit)).exit) return
  let timeElapsedInMinutes = Math.floor(moment(registro._DataRegistro).diff(ultimoRegistro._DataRegistro) / (1000 * 60))
  const hours = Math.floor(timeElapsedInMinutes / 60)
  const minutes = timeElapsedInMinutes % 60
  console.log(`${chalk.cyan('!')} Ponto de entrada realizado à ${blue.underline.bold.italic(hours + 'h ' + minutes + 'min')}`)
  const spinner = makeSpinner()
  const tabelaRegistros = new Table({head: ['Projeto', 'Tarefa', 'Descrição', 'Tempo']})
  try {
    while (timeElapsedInMinutes > 0) {
      const projeto = await promptProjeto(dominio.Projetos[0].Item)
      spinner.start(cyan(`Obtendo tarefas do projeto ${projeto._Descricao}...`))
      const {Tarefas} = await ucRegistrarPonto.obterTarefaProjeto(projeto._Codigo)
      spinner.succeed(`Tarefas do projeto ${projeto._Descricao} obtidas com ${green.bold.italic.underline('sucesso')}`)
      const tarefa = await promptTarefa(Tarefas)
      if (tarefa) {
        const description = await describeTask()
        const time = await timeTask(timeElapsedInMinutes)
        spinner.start(cyan(`Incluindo tarefa ${tarefa._Descricao}...`))
        await ucRegistrarPonto.incluirTarefa(tarefa._OID, time, description)
        tabelaRegistros.push([projeto._Descricao, tarefa._Descricao, description, timeToString(time)])
        spinner.succeed(`Tarefa ${cyan(tarefa._Descricao)} incluída com ${green.bold.italic.underline('sucesso')}`)
        timeElapsedInMinutes -= time
        if (timeElapsedInMinutes > 0) {
          const hours = Math.floor(timeElapsedInMinutes / 60)
          const minutes = timeElapsedInMinutes % 60
          console.log(`${chalk.cyan('!')} Restam ${blue.underline.bold.italic(hours + 'h ' + minutes + 'min')} a serem registradas`)
        }
      }
    }
    console.log(tabelaRegistros.toString())
    if ((await prompt(confirmExit)).confirmExit) await ucRegistrarPonto.confirmarRegistro()
  } catch (error) {
    if (error instanceof BusinessError) spinner.fail(chalk.bgRed.white.bold(error.message))
    else if (error instanceof ConnectionError) spinner.fail(chalk.bgRed.white.bold(error.message))
    else console.error(error)
  }
}

function timeToString (time: number) {
  const hours = Math.floor(time / 60)
  const hoursStr = hours ? hours + ' hora' + (hours > 1 ? 's' : '') : ''
  const minutes = time % 60
  const minutesStr = minutes ? minutes + ' minuto' + (minutes > 1 ? 's' : '') : ''
  return `${hoursStr}${(hours && minutes) ? ' e ' : ''}${minutesStr}`
}
